#ifndef _VisibilityBspSceneManager_H__
#define _VisibilityBspSceneManager_H__

#include <OgreSceneNode.h>
#include <OgreBspSceneManager.h>
#include <OgreOctreeNode.h>
#include <OgreOctreeCamera.h>
#include <OgrePrerequisites.h>

#include "OgreBspHierarchyInterface.h"
#include "GtpVisibility/Manager.h"

namespace Ogre {

/**
	This class extends the octree scene manager, using 
	occlusion queries for visibility culling.
*/
class VisibilityBspSceneManager: public BspSceneManager
{
public:
	VisibilityBspSceneManager(GtpVisibility::Manager *visManager);
	~VisibilityBspSceneManager();

	void _renderVisibleObjects();
	void _findVisibleObjects(Camera* cam, bool onlyShadowCasters);
	void _updateSceneGraph(Camera* cam );

	/** Sets the given option for the SceneManager
			    @remarks
		Options are:
		"Algorithm", int *;			
	*/
	
	virtual bool setOption(const String &, const void *);
	/** Gets the given option for the Scene Manager.
	    @remarks
		See setOption
	*/
	virtual bool getOption(const String &, void *);

	bool getOptionValues(const String & key, StringVector &refValueList);
	bool getOptionKeys(StringVector &refKeys);

	/** Sets the visibility manager.
		@param visManager the visibility manager
	*/
	void setVisibilityManager(GtpVisibility::Manager *visManager);
	/** See set.
	*/
	GtpVisibility::Manager *getVisibilityManager( void );

protected:
	
	BspHierarchyInterface *mHierarchyInterface;
	GtpVisibility::Manager *mVisibilityManager;
};

} // namespace Ogre

#endif // VisibilityBspSceneManager_H